%%Optimal charging strategy

function [totalcost, transitioncost, path_opt, p_bat_opt, SOC_opt] = ...
optimal_charging(p_load,p_pv,spotprice_no3, spot_timetariff,structure, ...
p_bat_max,eff_ch,eff_di,C_bat,soc_max,soc_min,delta_soc,year)
 %%Key parameters
 
%Number of possible SOCs. +1 because row 1 corresponding to SOC_min
 N_soc = 1 + (soc_max-soc_min)/delta_soc;
%Number of periods. 1 year = 8760
 N_periods = size(p_load,1)*size(p_load,2);

 %Maximum steps of change of SOC (integer)
 soc_max_change = floor(p_bat_max*eff_ch/(delta_soc*C_bat));
 
%%Grid tariff

 [fastledd, fastleddwithtax, energiledd, inntektsledd, ... 
    energiavgift, nettavgift,fastleddstrom, spot_timetariff, ...
    grunnfaktor,powerfactor,price_overuse,powerlimit] ...
        = getNettleie(year);
    
 %%Initialising arrays
 
 %Total minimum cost to finalise to last period
 totalcost = inf(N_soc,N_periods);

 %Costs to get from SOC [1st dim] in period [2nd dim] to SOC [3rd dim]
 %in period [2nd dim + 1]
 transitioncost = inf(N_soc,N_periods,N_soc); 
 
 %Gives which row (i.e. which SOC) that one should move to in period
 %[N_periods + 1] to min(costs)
 path_opt = zeros(N_soc,N_periods);

 %Optimum p_bat given an initial SOC.
 p_bat_opt = zeros(N_soc,N_periods);

 %Optimum SOC given an initial SOC.
SOC_opt = inf(N_soc,N_periods); 
 totalcost(1,N_periods) = 0; n
 transitioncost(1,N_periods,:) = 0; 

%Final state being SOC = 0.
soc_max_next = 1; %First row of SOC-array corresponding to soc_min
day = size(p_load,1); %365
hour = size(p_load,2); %24



%%Calculations

for i = 1:(N_periods-1)
    %%Decide possible SOC

    soc_max_i = min(N_soc,soc_max_next+soc_max_change);

    %%Set day and hour with respect to i

    if mod(N_periods-i,24) == 0
        hour = 24;
        day = day - 1;
    else
        hour = mod(N_periods-i,24);
    end

    %%Calculate costs
    
    for j = 1:soc_max_i 
    %Iterates over all legal soc's in period i
    %Iterates over all legal SOC's in period i+1 which can be reached
    %from SOC j in period t

        %Lowest reachable SOC from node j in period i
        soc_min_j = max(1,j-soc_max_change);
        %Highest reachable SOC from node j in period i
        soc_max_j = min(soc_max_next,j+soc_max_change);


        for k = soc_min_j:soc_max_j
            soc_change = (k-j)*delta_soc;
            %p_bat defined positive into battery
            p_bat_temp = soc_change*C_bat;
            if p_bat_temp > 0
                p_bat_temp = p_bat_temp/eff_ch;  %eff_ch < 1
            else
                p_bat_temp = p_bat_temp*eff_di;  %eff_di < 1
            end
                p_grid = p_load(day,hour) + p_bat_temp ... 
                        - p_pv(day,hour);
            if p_grid > 0
                switch structure
                    case 1
                        %Energy structure
                        transitioncost(j,N_periods-i,k) = ...
                        ((spotprice_no3(day,hour) + ...
                        inntektsledd) * 1.25 + nettavgift ...
                        + energiledd)*p_grid;
                    case 2
                        %Power tariff
                        transitioncost(j,N_periods-i,k) = ...
                        ((spotprice_no3(day,hour) + ... 
                        inntektsledd) * 1.25 + nettavgift) ...
                        * p_grid + ceil(p_grid) * powerfactor;
                    case 3
                        %Time of use, time tariff
                        transitioncost(j,N_periods-i,k) = ...
                        ((spotprice_no3(day,hour) + ...
                        inntektsledd) * 1.25 + nettavgift) ...
                        * p_grid + (spot_timetariff(day,hour)...
                        * p_grid);
                    case 4
                        %Subscription based tariff
                        if p_grid > powerlimit
                            transitioncost(j,N_periods-i,k) = ...
                            (p_grid - powerlimit) ...
                            * price_overuse;
                        end
                        transitioncost(j,N_periods-i,k) = ...
                        transitioncost(j,N_periods-i,k) + ...
                        (p_grid * ((spotprice_no3(day,hour)  ... 
                        + inntektsledd) * 1.25 + nettavgift));      
                end %switch
            else
                transitioncost(j,N_periods-i,k) = ...
                spotprice_no3(day,hour)*p_grid; %< 0
            end %if p_grid > 0 
        end %k

    %%Calculate SOC-value and find shortest path
    %All transition costs for given period and soc
    change_cost = transitioncost(j,N_periods-i,:);
    change_cost = squeeze(change_cost);

    totalcost_temp = totalcost(:,N_periods-i+1) + change_cost;
    %Minimum cost and transition
    [min_cost, min_place] = min(totalcost_temp);
    totalcost(j, N_periods-i) = min_cost;

    path_opt(j,N_periods-i) = min_place;
    end %j 
%%Preperation for new iteration
soc_max_next = soc_max_i;
end

%%Determining p_bat
for i = 1:N_soc
     soc_from = i;

    for j = 1:N_periods
        soc_to = path_opt(soc_from,j);
        p_bat_temp = (soc_to-soc_from)*C_bat*delta_soc;
            if p_bat_temp > 0
                p_bat_opt(i,j) = p_bat_temp/eff_ch;  
                % eff_ch < 0
            else
                p_bat_opt(i,j) = p_bat_temp*eff_di;  
                % eff_di < 0
            end
            SOC_opt(i,j) = (soc_from-1)*delta_soc;
            soc_from = soc_to;
    end
end
   p_bat_opt(:,N_periods) = 0;
end

